/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
ldse@hendriklipka.de
*/

package de.hendriklipka.ldse.server.data.document;

import java.io.*;
import java.net.URL;

import de.hendriklipka.ldse.common.Log;
import de.hendriklipka.ldse.server.data.document.Document;
import de.hendriklipka.ldse.server.indexer.DocumentFactory;
import de.hendriklipka.ldse.server.data.URLInfo;
import de.hendriklipka.ldse.common.Spider;

/**
  zerlegt ein PDF-Dokument in Worte
  Das Dokument wird in eine temporaere Datei geschrieben und dann mit
  "pdf2txt" in eine Textdatei umgewandelt
  Anschliessend wird ein Textdocument erzeugt, welches dann die eigentliche
  Zerlegung vornimmt
*/
public class PDFDocument extends Document implements Spider
{
  protected Reader _reader=null;
  protected InputStream _iStream=null;
  protected Document _doc=null;
  protected URL _url=null;
  protected String _name=null;
  
  public boolean init()
  {
		try
		{
		  String s=_urlInfo._url.getFile(); 
		  int pos=s.lastIndexOf('/');
		  if (-1!=pos)
		    s=s.substring(pos+1);
		  String infile=s+".pdf";
		  String outfile=s+".txt";
      Log.log("indexer","document",2,"converting pdf-document: "+infile+"->"+outfile);
		  _name=outfile;
		  String command="pdftotext -raw -q "+infile+" "+outfile;
		  // kopieren der Datei
		  // Streams, damit keine Umsetzung der Zeichen stattfindet (Binaerdateien!)
		  InputStream r=new BufferedInputStream(_urlInfo.getInputStreamForURL(_urlInfo._url),1000);
		  OutputStream wr=new BufferedOutputStream(new FileOutputStream(infile),1000);
		  int n=0;
		  while (true)
		  {
		    int c=r.read();
		    if (-1==c)
		      break;
		    wr.write(c);
		    n++;
		  }
		  r.close();
		  wr.close();
		  try
		  {
		    Process p=Runtime.getRuntime().exec(command);
		    int ret=p.waitFor();
		    if (0==ret)
		    {
		      _iStream=new FileInputStream(outfile);
		      _reader=new InputStreamReader(_iStream);
		    }
		    else
		    {
		      return false;
		    }
      }
      finally
      {
        File f=new File(infile);
        f.delete();
      }
      
      URL u=new URL(_urlInfo._url.getProtocol(),_urlInfo._url.getHost(),_urlInfo._url.getPort(),outfile);
      URLInfo ui=new URLInfo();
      ui._url=u;
      ui._contextURL=_urlInfo._contextURL;
      ui._urlStr=u.toString();
      ui._spiderID=_urlInfo._spiderID;
      ui._spider=this;
		  ui._type="text/plain";
		  // damit versorgen wir das neue Dokument mit den richtigen Daten, da wir selber
		  // ein Spider sind...
		  _url=u;
		  
		  _doc=DocumentFactory.getDocument(ui);
		  if (null!=_doc)
  		  return _doc.init();
      Log.log("indexer","document",2,"pdf-document: no document");
		}
		catch (Exception ex)
		{
	    ex.printStackTrace(Log.getWriter("document","error",0));
		  _doc=null;
		  return false;
		}
    return false;
  }
  
  public boolean hasMoreWords()
  {
    if (null==_doc)
    {
      return false;
    }
    return _doc.hasMoreWords();
  }
  
  public String nextWord()
  {
    if (null==_doc)
    {
      return null;
    }
    return _doc.nextWord();
  }
  
  public String getDescriptiveWords()
  {
    if (null==_doc)
    {
      return "";
    }
    String s=_doc.getDescriptiveWords();
    if (null==s)
      return "";
    return s;
  }
  
  public void finish()
  {
    try
    {
      if (null!=_doc)
      {
        _doc.finish();
      }
      if (null!=_reader)
      {
        _reader.close();
      }
      if (null!=_iStream)
      {
        _iStream.close();
      }
      if (null!=_name)
      {
        File f=new File(_name);
        Log.log("indexer","document",2,"pdf-document: trying to delete "+_name);
        boolean b=f.delete();
        if (!b)
        {
          Log.log("indexer","document",2,"pdf-document: can't delete tempfile: "+_name);
        }
      }
    }
    catch (Exception ex) 
    {
	    ex.printStackTrace(Log.getWriter("document","error",0));
    }
  }
  
  public java.io.Reader getReaderForURL(java.net.URL url) throws java.io.IOException
  {
    if (url==_url)
    {
      return _reader;
    }
    return null;
  }
  
  public java.io.InputStream getInputStreamForURL(java.net.URL url) throws java.io.IOException
  {
    if (url==_url)
    {
      return _iStream;
    }
    return null;
  }
  
  public void init(org.w3c.dom.Node configNode, int id){}
  public boolean hasMoreElements() { return false;}
  public URLInfo getNextElement() {return null;}
  public void start() {}
}