/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
ldse@hendriklipka.de
*/

package de.hendriklipka.ldse.server.gatherer;

import java.util.Vector;
import java.io.*;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Date;
import java.util.Hashtable;

import org.w3c.dom.*;

import de.hendriklipka.ldse.common.Util;
import de.hendriklipka.ldse.common.Log;
import de.hendriklipka.ldse.server.data.URLInfo;

public class FileSpider implements de.hendriklipka.ldse.common.Spider
{
  // Liste aller geladenen URLs. enthaelt URLInfos
  // begrenz, weil sie eh gleich wieder rausgeworfen werden...
  protected Vector _done = new Vector(500,500);
  protected String _urlPrefix=null;
  protected String _start=null;
  protected Runner _runner=null;

// Parameter fuer die Suche
  protected int _maxDepth=30;
  protected int _maxQueue=50;
  protected String[] _pathExclude=null;
  protected Hashtable _types=new Hashtable();

  protected int _id=-1;

  public void init(org.w3c.dom.Node configNode, int id)
  {
    // Standardwert fuer Dateityp:
    _types.put("*","text/html");
    _id=id;
    NodeList nl=configNode.getChildNodes();
    for (int i=0;i<nl.getLength();i++)
    {
      Node node=nl.item(i);
      if (node.getNodeType()!=node.ELEMENT_NODE)
        continue;
      String nn=node.getNodeName();
      if (nn.equals("depth"))
      {
        _maxDepth=Integer.parseInt(de.hendriklipka.ldse.common.Util.getTextData(node));
        Log.log("spider","config",1,"depth="+_maxDepth);
      }
      else if (nn.equals("urlprefix"))
      {
        _urlPrefix=de.hendriklipka.ldse.common.Util.getTextData(node);
      }
      else if (nn.equals("maxqueue"))
      {
        String s=de.hendriklipka.ldse.common.Util.getTextData(node);
        _maxQueue=Integer.parseInt(s);
      }
      else if (nn.equals("start"))
      {
        try
        {
          String urlStr=de.hendriklipka.ldse.common.Util.getTextData(node);
          Log.log("spider","config",1,"start="+urlStr);
          _start=urlStr;
        }
        catch (Exception ex)
        {
          Log.log("spider","error",0,ex.toString());
        }
      }
      else if (nn.equals("pathexclude"))
      {
        _pathExclude=de.hendriklipka.ldse.common.Util.stringToArray(de.hendriklipka.ldse.common.Util.getTextData(node));
      }
      else if (nn.equals("filetypes"))
      {
        String data="";
        NodeList subs=node.getChildNodes();
        for (int j=0;j<subs.getLength();j++)
        {
          Node attr=subs.item(j);
          if (attr.getNodeType()!=attr.TEXT_NODE)
            continue;
          Text val=(Text)attr;
          String s=val.getData().trim();
          if (0!=s.length())
          {
            data+=" "+s;
          }
        }
        data=data.trim();
        String[] sa=de.hendriklipka.ldse.common.Util.stringToArray(data);
        for (int j=0;j<sa.length;j++)
        {
          String name=sa[j].trim();
          int pos=name.indexOf('=');
          if (-1==pos)
            continue;
          String value=name.substring(pos+1);
          name=name.substring(0,pos);
          _types.put(name,value);
        }
      }
      else
      {
        Log.log("spider","error",1,"unknown node - type="+node.getNodeType()+" name="+node.getNodeName()+" value="+node.getNodeName());
      }
    }
  }
  
  // ToDo: fertig implementieren
  public boolean hasMoreElements()
  {
    // fertige URLs
    if (0!=_done.size())
    {
      return true;
    }
    // wird immer von anderem Thread als dem des Spiders aus aufgerufen
    if (null!=_runner)
    {
      if (_runner.isAlive())
      {
        return true;
      }
    }
    return false;
  }
  
  // ToDo: fertig implementieren
  public URLInfo getNextElement()
  {
    // erstmal fertige URLs liefern
    synchronized (_done)
    {
      // Jetzt koennen wieder Elemente eingefuegt werden
      if (_done.size()<_maxQueue/2)
      {
        Log.log("spider","threads",1,"restarting thread waiting for queue");
        _done.notifyAll();
      }
      if (!_done.isEmpty())
      {
        URLInfo ui=(URLInfo)_done.firstElement();
        _done.removeElementAt(0);
        return ui;
      }
      if ((null!=_runner)&&(_runner.isAlive()))
      {
        try
        {
          _done.wait();
        }
        catch (InterruptedException ex) {}
        if (!_done.isEmpty())
        {
          URLInfo ui=(URLInfo)_done.firstElement();
          _done.removeElementAt(0);
          return ui;
        }
      }
    }
    // wird immer von anderem Thread als dem des Spiders aus aufgerufen
    
    return null;
  }
  
  public void start()
  {
    _runner=new Runner(_start);
    _runner.start();
  }
  
  // _urlPrefix abziehen und _prefixFile bzw. Start anfuegen
  // dann Reader aufmachen
  public java.io.Reader getReaderForURL(java.net.URL url) throws java.io.IOException
  {
    String s=url.toString();
    int i=s.indexOf(_urlPrefix);
    if (-1==i)
      i=0;
    s=s.substring(_urlPrefix.length()+i);
    s=_start+s;
    return new FileReader(s);
  }
  
  public java.io.InputStream getInputStreamForURL(java.net.URL url) throws java.io.IOException
  {
    String s=url.toString();
    int i=s.indexOf(_urlPrefix);
    if (-1==i)
      i=0;
    s=s.substring(_urlPrefix.length()+i);
    s=_start+s;
    return new FileInputStream(s);
  }

  protected void putFile(File file, String fName)
  {
    synchronized(_done)
    {
      if (_done.size()>_maxQueue)
      {
        // Blockieren
        Log.log("spider","threads",1,"stopping because queue to large");
        try
        {
          _done.wait();
        }
        catch (InterruptedException ex) {}
        Log.log("spider","threads",1,"restarting this thread");
      }
      if (null!=_pathExclude)
      {
        String name=file.getAbsolutePath();
        for (int i=0;i<_pathExclude.length;i++)
        {
          if (-1!=name.indexOf(_pathExclude[i]))
          {
            return;
          }
        }
      }
      URLInfo ui=new URLInfo();
      String full=_urlPrefix+fName;
      ui._urlStr=full;
      try
      {
        ui._url=new URL(full);
      }
      catch (MalformedURLException ex) {}
      String ext="";
      int pos=fName.lastIndexOf('.');
      if (-1!=pos)
      {
        ext=fName.substring(pos+1);
      }
      if (0==ext.length())
      {
        ext=".";
      }
      String type=(String)_types.get(ext);
      if (null==type)
      {
        type=(String)_types.get("*");
      }
      ui._spiderID=_id;
      ui._size=(int)file.length();
      ui._lastChanged=new Date(file.lastModified());
      ui._type=type;
      ui._contextURL=null; // Kontext haben wir nicht
      ui._depth=0; // Tiefe ist uninteressant
      ui._checksum=0; // Checksumme muesste erst berechnet werden -> ToDo
      ui._spider=this;
      ui._parent=null; // kein Parent
      Log.log("spider","url",2,"added file as URL: "+full+" as type: "+type);
      _done.addElement(ui);
      _done.notifyAll();
    }
  }
  
  private class Runner extends Thread
  {
    protected String _start=null;
    
    public Runner(String start)
    {
      super("FileSpider.Runner");
      _start=start;
    }
    
    public void run()
    {
      File file=new File(_start);
      if (!file.isDirectory())
      {
        file=new File(file.getParent());
      }
      Log.log("spider","url",1,"starting search from: "+file.getAbsolutePath());
      listFiles(file,_maxDepth,"");
      synchronized(_done)
      {
        _runner=null;
        _done.notifyAll();
      }
    }
    
    protected void listFiles(File file, int depth, String pathPrefix)
    {
      if (0==depth)
      {
        Log.log("spider","url",1,"exceeding max depth: ");
        return;
      }
      String[] files=file.list();
      for (int i=0;i<files.length;i++)
      {
        File f=new File(_start+pathPrefix+files[i]);
        if (f.isFile())
        {
          if (f.canRead())
          {
            Log.log("spider","url",2,"found file: "+files[i]);
            putFile(f,pathPrefix+files[i]);
          }
          else
          {
            Log.log("spider","url",0,"found file, but cannot read: "+files[i]);
          }
        }
        else if (f.isDirectory())
        {
          if ((files[i].equals("."))||(files[i].equals("..")))
          {
            Log.log("spider","url",3,"found . or ..: ");
            continue;
          }
          Log.log("spider","url",2,"diving into: "+files[i]);
          listFiles(f,depth-1,pathPrefix+files[i]+"/");
        }
        else
        {
          Log.log("spider","url",0,"found file, which is not dir nor file: "+files[i]+" -> "+f.toString());
        }
      }
    }
  }
}