/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
ldse@hendriklipka.de
*/

package de.hendriklipka.ldse.server.query;

import org.w3c.dom.*;

import com.objectspace.jgl.Container;

import de.hendriklipka.ldse.server.dataserver.DBConnection;
import de.hendriklipka.ldse.common.ResultDescriptor;
import de.hendriklipka.ldse.server.broker.BrokerQuery;

public abstract class QueryProcessor
{
  protected com.objectspace.jgl.Container _result=null;
  protected Node _query=null;
  protected QueryLimits _limits=null;
  
  public QueryProcessor(Node query)
  {
    _query=query;
  }
  
  public void setLimits(QueryLimits limits)
  {
    _limits=limits;
  }
  
  public com.objectspace.jgl.Container getResult()
  {
    return _result;
  }
  
  /**
    Initialisierung
  */
  public abstract void init();
  
  /**
    Vorverarbeitung der Query
  */
  public abstract void preprocessQuery();
  
  /**
    Abfrage der Datenbank
  */
  public abstract void doQuery(DBConnection con);
  
  /**
    Informationen aus der DB zum eigentlichen Result verarbeiten
  */
  public abstract void postprocessQuery(DBConnection con);
  
  public abstract BrokerQuery createBrokerQuery();
  
  public abstract void close();
}